from rich.console import Console
from rich.progress import (
    Progress,
    TextColumn,
    BarColumn,
    MofNCompleteColumn,
    SpinnerColumn,
    TimeElapsedColumn,
    TimeRemainingColumn,
)

from camiskaze.domain.messages import TimesheetRetrieved, TimeEntryRegistered


_LOG_TEMPLATE = "{mark} {work_order} - {date} - {description} - {project}"
_COLUMNS = [
    TextColumn(
        "[bold green]Registering Time Entries in Camis",
        justify="right",
    ),
    SpinnerColumn("clock"),
    BarColumn(),
    "[progress.percentage]{task.percentage:>3.1f}%",
    "•",
    MofNCompleteColumn(),
    "•",
    TimeRemainingColumn(),
    "•",
    TimeElapsedColumn(),
]
_DEFAULT_TIME_FORMAT = "[%Y-%m-%d %H:%M:%S.%f]"


class ProgressAdapter:
    _task_id: int

    def __init__(self, console: Console = None):
        self.console = console or Console(log_time_format=_DEFAULT_TIME_FORMAT)

        self.progress = Progress(*_COLUMNS, console=self.console)

    def start(self, event: TimesheetRetrieved) -> None:
        self.progress.start()

        self._task_id = self._add_task(event.num_time_entries)

    def _add_task(self, total: int) -> int:
        return self.progress.add_task(
            description="registration.",
            total=total,
        )

    def update(self, event: TimeEntryRegistered) -> None:
        self._log(event)
        self.progress.update(self._task_id, advance=1)

    def _log(self, event: TimeEntryRegistered) -> None:
        log_string = _LOG_TEMPLATE.format(
            mark=_get_mark(event),
            description=event.description,
            work_order=event.work_order,
            date=event.date,
            project=event.project,
        )
        self.progress.log(log_string)

    def stop(self) -> None:
        self.progress.stop()


def _get_mark(event: TimeEntryRegistered) -> str:
    if event.success:
        return ":white_check_mark:"

    return ":skull:"
