from aiohttp import ClientSession

from camiskaze.adapters.exceptions import WorkOrderCacheMissError

from . import TogglProjectRepository


class TogglWorkOrderRepository:
    _projects: TogglProjectRepository
    _cache: dict[str, tuple[str | None, str]]
    _cache_key_template = "{workspace_id}:{project_id}"

    def __init__(self, session: ClientSession):
        self.projects = TogglProjectRepository(session)

        self._cache = {}

    async def get(self, workspace_id: int, project_id: int) -> str:
        try:
            return self._get_from_cache(workspace_id, project_id)
        except WorkOrderCacheMissError as error:
            work_order, project_name = await self._get(
                workspace_id,
                project_id,
            )

            self.add_to_cache(error.key, work_order, project_name)

            return work_order, project_name

    def _get_from_cache(
        self,
        workspace_id: int,
        project_id: int,
    ) -> str:
        key = self._build_cache_key(workspace_id, project_id)

        try:
            return self._cache[key]
        except KeyError:
            raise WorkOrderCacheMissError(key)

    async def _get(
        self,
        workspace_id: int,
        project_id: int,
    ) -> tuple[str | None, str]:
        project = await self.projects.get(workspace_id, project_id)

        return project.work_order, project.name

    def add_to_cache(
        self,
        key: str,
        work_order: str,
        project_name: str,
    ) -> None:
        self._cache[key] = (work_order, project_name)

        return work_order

    def _build_cache_key(self, workspace_id: int, project_id: int) -> str:
        return self._cache_key_template.format(
            workspace_id=workspace_id,
            project_id=project_id,
        )
