from aiohttp import BasicAuth

from camiskaze.config import camis_config, toggl_config
from camiskaze.domain.messages import (
    Register,
    RetrieveTimesheet,
    RegisterTimesheet,
    RegisterTimeEntry,
    TimesheetRetrieved,
    TimesheetRetrievalStarted,
    TimeEntryRegistered,
    TimesheetRegistered,
    TimesheetGrouped,
)
from camiskaze.service_layer import UnitOfWork, MessageBus
from camiskaze.service_layer.handlers import (
    register,
    register_timesheet,
    retrieve_timesheet,
    start_progress,
    update_progress,
    group_timesheet,
    log_retrieval,
    log_timesheet_registered,
    log_timesheet_retrieval_started,
    register_time_entry,
    stop_status,
    update_table,
)


async def setup() -> None:
    pass


def create_message_bus() -> MessageBus:
    message_bus = MessageBus()

    message_bus.register(Register, register)
    message_bus.register(RetrieveTimesheet, retrieve_timesheet)
    message_bus.register(RegisterTimesheet, register_timesheet)
    message_bus.register(TimesheetRetrieved, log_retrieval)
    message_bus.register(TimesheetRetrieved, group_timesheet)
    message_bus.register(TimesheetRetrieved, start_progress)
    message_bus.register(TimesheetGrouped, stop_status)
    message_bus.register(
        TimesheetRetrievalStarted,
        log_timesheet_retrieval_started,
    )
    message_bus.register(RegisterTimeEntry, register_time_entry)
    message_bus.register(TimeEntryRegistered, update_progress)
    message_bus.register(TimeEntryRegistered, update_table)
    message_bus.register(TimesheetRegistered, log_timesheet_registered)

    return message_bus


def create_unit_of_work(toggl_user: str) -> UnitOfWork:
    return UnitOfWork(
        camis_base_url=camis_config.URL,
        camis_auth=_create_camis_auth(),
        camis_headers=_create_camis_headers(),
        toggl_base_url=toggl_config.URL,
        toggl_auth=_create_toggl_auth(toggl_user),
        toggl_headers=_create_toggl_headers(),
    )


def _create_camis_auth() -> BasicAuth:
    return BasicAuth(
        login=camis_config.USERNAME,
        password=camis_config.PASSWORD,
    )


def _create_toggl_auth(toggl_user: str) -> BasicAuth:
    return BasicAuth(
        login=toggl_user,
        password=toggl_config.get_password(toggl_user),
    )


def _create_toggl_headers() -> dict:
    app_json = "application/json"

    return {
        "Accept": app_json,
        "Content-Type": app_json,
    }


def _create_camis_headers() -> dict:
    app_json = "application/json"

    return {
        "Accept": app_json,
        "Content-Type": app_json,
    }
