from camiskaze.adapters.exceptions import RegistrationError
from camiskaze.domain.messages.commands import RegisterTimeEntry
from camiskaze.domain.messages.events import TimeEntryRegistered
from camiskaze.service_layer import UnitOfWork, MessageBus


async def register_time_entry(
    command: RegisterTimeEntry,
    unit_of_work: UnitOfWork,
    message_bus: MessageBus,
) -> None:
    registrator = TimeEntryRegistrator(command, unit_of_work, message_bus)

    await registrator.register()


class TimeEntryRegistrator:
    max_retries = 3

    def __init__(
        self,
        command: RegisterTimeEntry,
        unit_of_work: UnitOfWork,
        message_bus: MessageBus,
    ):
        self.command = command
        self.unit_of_work = unit_of_work
        self.message_bus = message_bus
        self.time_entry = command.time_entry

        self.num_retries = 0

    async def register(self) -> None:
        if not self.time_entry.work_order:
            await self._log_failed_completion(
                "No Work Order found for Time Entry"
            )
        else:
            await self._register()

    async def _register(self, error: Exception = None) -> None:
        if self.num_retries >= self.max_retries:
            await self._log_failed_completion(str(error))
        else:
            await self.__register()

    async def __register(self) -> None:
        try:
            await self.unit_of_work.camis.register(self.command)
        except RegistrationError as error:
            self.num_retries += 1
            await self._register(error)
        else:
            await self._log_successful_completion()

    async def _log_failed_completion(self, error: str) -> None:
        event = TimeEntryRegistered(
            time_entry=self.time_entry,
            success=False,
            reason=error,
        )
        await self.message_bus.publish(event, self.unit_of_work)

    async def _log_successful_completion(self) -> None:
        event = TimeEntryRegistered(
            time_entry=self.time_entry,
            success=True,
            reason="",
        )
        await self.message_bus.publish(event, self.unit_of_work)
