from aiohttp import BasicAuth, ClientSession

from rich.console import Console

from camiskaze.adapters import (
    CamisTimeSheetRepository,
    TableAdapter,
    ProgressAdapter,
    StatusAdapter,
    TogglTimeSheetRepository,
)


class UnitOfWork:
    camis: CamisTimeSheetRepository
    progress: ProgressAdapter
    status: StatusAdapter
    table: TableAdapter
    toggl: TogglTimeSheetRepository

    def __init__(
        self,
        camis_base_url: str,
        camis_auth: BasicAuth,
        camis_headers: dict,
        toggl_base_url: str,
        toggl_auth: BasicAuth,
        toggl_headers: dict,
    ):
        self.camis_base_url = camis_base_url
        self.camis_auth = camis_auth
        self.camis_headers = camis_headers
        self.toggl_base_url = toggl_base_url
        self.toggl_auth = toggl_auth
        self.toggl_headers = toggl_headers

    async def __aenter__(self) -> "UnitOfWork":
        self._create_camis()
        self._create_output()
        self._create_toggl()

        return self

    def _create_camis(self) -> None:
        self._camis_session = ClientSession(
            base_url=self.camis_base_url,
            auth=self.camis_auth,
            headers=self.camis_headers,
        )

        self.camis = CamisTimeSheetRepository(self._camis_session)

    def _create_output(self) -> None:
        self._create_progress()
        self._create_status()
        self._create_table()

    def _create_progress(self) -> None:
        self.progress = ProgressAdapter()

    def _create_status(self) -> None:
        self.status = StatusAdapter()

    def _create_table(self) -> None:
        self.table = TableAdapter()

    def _create_toggl(self) -> None:
        self._toggl_session = ClientSession(
            base_url=self.toggl_base_url,
            auth=self.toggl_auth,
            headers=self.toggl_headers,
        )

        self.toggl = TogglTimeSheetRepository(self._toggl_session)

    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
        await self._camis_session.close()
        await self._toggl_session.close()
